package com.m2pool.system.service.impl;

import com.m2pool.common.core.constant.UserConstants;
import com.m2pool.common.core.exception.ServiceException;
import com.m2pool.common.core.utils.GoogleAuthenticator;
import com.m2pool.common.core.utils.SpringUtils;
import com.m2pool.common.core.utils.StringUtils;
import com.m2pool.common.core.web.Result.AjaxResult;
import com.m2pool.common.datascope.annotation.DataScope;
import com.m2pool.common.security.utils.SecurityUtils;
import com.m2pool.system.api.entity.SysRole;
import com.m2pool.system.api.entity.SysUser;
import com.m2pool.system.api.entity.SysUserLeveDate;
import com.m2pool.system.entity.GoogleInfo;
import com.m2pool.system.entity.SysUserRole;
import com.m2pool.system.mapper.SysRoleMapper;
import com.m2pool.system.mapper.SysUserLevelMapper;
import com.m2pool.system.mapper.SysUserMapper;
import com.m2pool.system.mapper.SysUserRoleMapper;
import com.m2pool.system.service.SysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Description 用户业务层
 * @Date 2024/6/14 10:37
 * @Author dy
 */
@Service
public class SysUserServiceImpl implements SysUserService {


    private static final Logger log = LoggerFactory.getLogger(SysPermissionServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysUserLevelMapper userLevelMapper;


    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(SysUser user)
    {
        return userMapper.selectUserList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName) {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过邮箱查询用户
     *
     * @param email 邮箱
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByEmail(String email) {
        return userMapper.selectUserByEmail(email);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId)
    {
        return userMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    @Override
    public String checkUserNameUnique(String userName) {
        int count = userMapper.checkUserNameUnique(userName);
        if (count > 0)
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkPhoneUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhone());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkEmailUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否存在
     *
     * @param email 邮箱
     * @return
     */
    @Override
    public String checkEmailExist(String email)
    {
        String dbEmail = userMapper.checkEmailExist(email);
        if (StringUtils.isNotNull(dbEmail))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user)
    {
        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
        {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(Long userId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysUser user = new SysUser();
            user.setUserId(userId);
            List<SysUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
            if (StringUtils.isEmpty(users))
            {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }


    @Override
    public boolean registerUser(SysUser user) {

        boolean result = userMapper.insertUser(user) > 0;
        if(result){
            //用户添加成功后 默认设置用户为注册用户
            SysUser dbUser = userMapper.selectUserByUserName(user.getUserName());
            user.setUserId(dbUser.getUserId());

            //查询角色：“注册用户”的id并且赋值到user中
            Long[] roleIds = roleMapper.getResigesterId();
            //先把注册用户等级都升为vip 有效期三个月(后通过定时任务去控制)
            //Long[] roleIds = roleMapper.getVipId();
            user.setRoleIds(roleIds);
            insertUserRole(user);

        }

        return result;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateUser(SysUser user)
    {
        //Long userId = user.getUserId();

        System.out.println("修改用户信息"+user);
        // 删除用户与角色关联
        //userRoleMapper.deleteUserRoleByUserId(userId);
        //// 新增用户与角色管理  此处要保证传过来的user中要携带roleIds
        //insertUserRole(user);
        //// 删除用户与岗位关联
        //userPostMapper.deleteUserPostByUserId(userId);
        //// 新增用户与岗位管理
        //insertUserPost(user);
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 重置用户密码
     *
     * @param email 邮箱
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetPwdByEmail(String email, String password)
    {
        return userMapper.resetPwdByEmail(email, password);
    }

    @Override
    public int userLevelUp(SysUserLeveDate userInfo) {
        System.out.println("用户升级服务收到用户信息："+userInfo);
        Long userId = userInfo.getUserId();
        if(userInfo.getOper() == 1){
            //执行续费操作  用户角色不改变  会员过期日期在原有基础上增加
            //1、查询会员有效期
            SysUserLeveDate dbUserInfo = userLevelMapper.getUserLevelInfo(userId);
            Date date = dbUserInfo.getExpireTime();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            if(userInfo.getLevelType() > 0){
                //年度会员/年度vip  会员有效期设置为一年
                calendar.add(Calendar.YEAR,1);
            }else {
                calendar.add(Calendar.MONTH,1);
            }
            //2、根据购买类型修改会员有效期
            userInfo.setStartTime(dbUserInfo.getStartTime());
            userInfo.setExpireTime(calendar.getTime());


            userLevelMapper.deleteUserLevelInfoByUserId(userInfo.getUserId());
            // 新增用户与用户会员有效期关联
            return userLevelMapper.createUserLevelInfo(userInfo);

        }else{
            //执行升级操作  用户角色改变  会员过期日期在更新

            // 删除用户与角色关联
            userRoleMapper.deleteUserRoleByUserId(userId);
            // 新增用户与角色管理  此处要保证传过来的user中要携带roleIds
            if (StringUtils.isNotNull(userInfo.getRoleId()))
            {
                // 新增用户与角色关联
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userInfo.getUserId());
                ur.setRoleId(userInfo.getRoleId());
                int userRole = userRoleMapper.createUserRole(ur);
                System.out.println("用户角色修改结果："+userRole);
                //获取用户当前会员有效期信息 根据情况决定如何处理
                // 相同类型则表示续费、直接增加日期 不同类型则直接删除有效期再更新
                // 删除用户与用户会员有效期关联
                userLevelMapper.deleteUserLevelInfoByUserId(userInfo.getUserId());
                // 新增用户与用户会员有效期关联
                return userLevelMapper.createUserLevelInfo(userInfo);
            }
        }
        return 0;
    }

    @Override
    public boolean checkGoogleCode(Long code) {
        String username = SecurityUtils.getUsername();

        //谷歌验证码校验
        GoogleInfo info = userMapper.getGoogleInfoByEmail(username);
        if(StringUtils.isNull(info)){
            throw new ServiceException("网络异常,请稍后再试");
        }

        //没有绑定谷歌验证器 校验直接通过
        if(StringUtils.isBlank(info.getSecret())){
            return true;
        }

        //没有绑定谷歌验证器 校验直接通过
        if(info.getStatus() == 0){
            return true;
        }

        if(GoogleAuthenticator.checkCode(info.getSecret(), code, System.currentTimeMillis())){
            return true;
        }else {
            throw new ServiceException("验证码错误");
        }
    }

    @Override
    public List<SysUser> getCSList() {

        List<SysUser> list = userMapper.getCSList();

        return list;
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user)
    {
        //需要确保user.getRoleIds在整个传递过程中不丢失
        Long[] roles = user.getRoleIds();
        if (StringUtils.isNotNull(roles))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>();
            for (Long roleId : roles)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(user.getUserId());
                ur.setRoleId(roleId);
                list.add(ur);
            }
            if (list.size() > 0)
            {
                userRoleMapper.batchUserRole(list);
            }
        }
    }


}
